comment {
// listCountObj
//
// this function gets an array and returns an array
// it basically counts every occurence of each element
// and creates an array of this
//
// PARAMS:
//    _this:  Array
// RETURNS:
//    _ret: [[obj0, count0], [obj1, count1], ...]
// NOTE:
//    creates a copy of the input list, thus may be CPU intensive.
// EXAMPLE:
//    ["a", "a", "b", "c", "c", "d", "e", "e"] call f_listCountObj
//    returns: [["a",2],["b",1],["c",2],["d",1],["e",2]]
};

f_listCountObj = {
   private["_list","_ret","_o"];

   _list = +_this;
   _ret = [];

   while {count _list > 0} do {
      _o = _list select 0;
      _ret set [count _ret, [_o, {_o == _x} count _list]];
      _list = _list - [_o];
   };
   _ret
};


comment {
// listFilter
//
// this function takes a list and returns a list that was filtered by the filter function
// all entries, that do not satisfy the filter condition will be thrown out of the list
// PARAMS:
//    _this[0]:  list to be filtered
//    _this[1]:  filter function
// RETURNS:
//    _ret:  filtered function
// EXAMPLE:
//    [mylist, {alive _this}] call f_listFilter
};

f_listFilter = {
   private["_list","_func","_c","_i","_v","_ret"];
   _list = _this select 0;
   _func = _this select 1;
   _c = count _list;
   _i = -1;
   _ret = [];
   while {_i=_i+1; _i < _c} do {
      _v = _list select _i;
      if( _v call _func ) then {
         _ret set[count _ret, _v];
      };
   };
   _ret
};


comment {
// listMap
//
// the famous Map-function, which takes a list and a function
// and creates a list of all values resulting from function applied to x
// The function has _this as its value: ex.: func = {typeOf _this}
//
// PARAMS:
//    _this[0] :  list
//    _this[1] :  function
// RETURNS:
//   _ret:  [(list select 0) call function, (list select 1) call function, ... ]
// EXAMPLE:
//    _new = [_mylist, {sin _this}] call f_listMap
};

f_listMap = {
   private["_list","_func","_c","_i","_ret","_x"];
   _list = _this select 0;
   _func = _this select 1;
   _c = count _list;
   _i = -1;
   _ret = [];
   while {_i=_i+1; _i < _c} do {
      _ret set [_i, (_list select _i) call _func];
   };
   _ret
};


comment {
// listSubArray
//
// returns a sub-array of the array
//
// PARAM:
//    _this[0]: array
//    _this[1]: lower index
//    _this[2]: count
// RETURNS:
//    _ret: [array[lowerIndex], array[lowerIndex+1], ..., array[lowerIndex+count-1]]
// DEPENDS:
//    varname: description
// EXAMPLE:
//    example
};

f_listSubArray = {
   private["_arr","_idx","_c","_res","_i","_n"];
   _arr = _this select 0;
   _idx = _this select 1;
   _c = _this select 2;
   
   _res = [];
   _i=_idx-1;
   _n=_idx+_c;
   while{_i=_i+1;_i<_n} do {
      _res set[count _res, _arr select _i];
   };
   _res
};


comment {
// listInsert
//
// inserts an item at a specific position and moves all other entries one position to the back.
// this function modifies the passed array, it does not return a new array.
//
// PARAM:
//    _this[0]: array
//    _this[1]: index/position at which the item should be inserted
//    _this[2]: item to be inserted
// EXAMPLE:
//    A = [1, 2, 3, 4, 5];
//    [A, 2, 10] call f_listInsert;
//    -> A is now [1, 2, 10, 3, 4, 5]
};
f_listInsert = {
   _arr = _this select 0;
   _idx = _this select 1;
   _item = _this select 2;
   
   _i = count _arr + 1;
   while{_i=_i-1; _i > _idx} do {
      _arr set[_i, _arr select (_i-1)];
   };
   _arr set[_idx, _item];
};

comment {
// listUniquify
//
// returns a new array in which all items occur exactly ones
// (it removes all duplicates)
//
// PARAM:
//    _this: an array
// RETURNS:
//    _ret: an array without duplicates
// EXAMPLE:
//    _a = [1, 4, 4, 1, 3] call f_listUniquify;
//    -> _a is [1, 4, 3]
};
f_listUniquify = {
   _ret = [];
   {
      if !(_x in _ret) then { _ret set[count _ret, _x] }
   } foreach _this;
   _ret
};